@@ -563,7 +563,8 @@ read_portage_profile(const char *profile, env_vars vars[], set *masks)
 					repo_name = NULL;
 				}
 				if (repo_name == NULL) {
-					warn("ignoring parent with unknown repo in profile: %s", s);
+					warn("ignoring parent with unknown repo in profile %s: %s",
+							profile, s);
 					continue;
 				}
 			} else {
